﻿/*
 Copyright aswing.org, see the LICENCE.txt.
*/

import org.aswing.CellEditor;
import org.aswing.Component;
import org.aswing.Container;
import org.aswing.event.CellEditorListener;
import org.aswing.geom.Rectangle;
import org.aswing.util.ArrayUtils;

/**
 * @author iiley
 */
class org.aswing.AbstractCellEditor implements CellEditor{
	
	private var listeners:Array;
	private var clickCountToStart:Number;
	
	public function AbstractCellEditor(){
		listeners = new Array();
		clickCountToStart = 0;
	}
	
    /**
     * Specifies the number of clicks needed to start editing.
     * Default is 0.(mean start after pressed)
     * @param count  an int specifying the number of clicks needed to start editing
     * @see #getClickCountToStart()
     */
    public function setClickCountToStart(count:Number):Void {
		clickCountToStart = count;
    }

    /**
     * Returns the number of clicks needed to start editing.
     * @return the number of clicks needed to start editing
     */
    public function getClickCountToStart():Number {
		return clickCountToStart;
    }	
    
    public function getEditorComponent():Component{
		trace("Error: Subclass should override this method!");
		throw new Error("Subclass should override this method!");
		return null;
    }
	
	public function getCellEditorValue() {		
		trace("Error: Subclass should override this method!");
		throw new Error("Subclass should override this method!");
	}
	
   /**
    * Sets the value of this cell. 
    * @param value the new value of this cell
    */
	public function setValue(value):Void{		
		trace("Error: Subclass should override this method!");
		throw new Error("Subclass should override this method!");
	}

	public function isCellEditable(clickCount : Number) : Boolean {
		return clickCount == clickCountToStart;
	}

	public function startCellEditing(owner : Container, value, bounds : Rectangle) : Void {
		var com:Component = getEditorComponent();
		setValue(value);
		com.removeFromContainer();
		com.addTo(owner);
		com.setBounds(bounds);
	}

	public function stopCellEditing() : Boolean {
		removeEditorComponent();
		fireEditingStopped();
		return true;
	}

	public function cancelCellEditing() : Void {
		removeEditorComponent();
		fireEditingCanceled();
	}

	public function addCellEditorListener(l : CellEditorListener) : Void {
		listeners.push(l);
	}

	public function removeCellEditorListener(l : CellEditorListener) : Void {
		ArrayUtils.removeFromArray(listeners, l);
	}
	
	private function fireEditingStopped():Void{
		for(var i:Number = listeners.length-1; i>=0; i--){
			var l:CellEditorListener = CellEditorListener(listeners[i]);
			l.editingStopped(this);
		}
	}
	private function fireEditingCanceled():Void{
		for(var i:Number = listeners.length-1; i>=0; i--){
			var l:CellEditorListener = CellEditorListener(listeners[i]);
			l.editingCanceled(this);
		}
	}
	
	private function removeEditorComponent():Void{
		var com:Component = getEditorComponent();
		com.removeFromContainer();
	}
	
	/**
	 * When an action is performed, editing is ended.
	 * @param e the action event
	 * @see #stopCellEditing
	 */
    public function actionPerformed(source:Component):Void {
		stopCellEditing();
	}
}